/*
 * This Source Code is subject to the terms of the Mozilla Public License
 * version 2.0 (the "License"). You can obtain a copy of the License at
 * http://mozilla.org/MPL/2.0/.
 */

const {classes: Cc, interfaces: Ci, utils: Cu} = Components;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");

var disabled = true;
const PREFBRANCH = "extensions.{9dfaef2c-b772-4bde-b5fc-1f69bd105c17}.";

/* Includes a javascript file with loadSubScript
*
* @param src (String)
* The url of a javascript file to include.
*/
(function(global) global.include = function include(src) {
  var o = {};
  Components.utils.import("resource://gre/modules/Services.jsm", o);
  var uri = o.Services.io.newURI(
      src, null, o.Services.io.newURI(__SCRIPT_URI_SPEC__, null, null));
  o.Services.scriptloader.loadSubScript(uri.spec, global);
})(this);

/* Imports a commonjs style javascript file with loadSubScrpt
 * 
 * @param src (String)
 * The url of a javascript file.
 */
(function(global) {
  var modules = {};
  global.require = function require(src) {
    if (modules[src]) return modules[src];
    var scope = {require: global.require, exports: {}};
    var tools = {};
    Cu.import("resource://gre/modules/Services.jsm", tools);
    var baseURI = tools.Services.io.newURI(__SCRIPT_URI_SPEC__, null, null);
    try {
      var uri = tools.Services.io.newURI(
          "packages/" + src + ".js", null, baseURI);
      tools.Services.scriptloader.loadSubScript(uri.spec, scope);
    } catch (e) {
      var uri = tools.Services.io.newURI(src, null, baseURI);
      tools.Services.scriptloader.loadSubScript(uri.spec, scope);
    }
    return modules[src] = scope.exports;
  }
})(this);

include("includes/buttons.js");

var {unload} = require("unload");
var {runOnLoad, runOnWindows, watchWindows} = require("window-utils");
var {loadOverlay, applyOverlay} = require("overlay");

// Listen for unloader notifications
var observer = {
	observe: function(aSubject, aTopic, aData)
	{
		switch (aTopic) {
		case "dslr-notifier-unload":
			if ((typeof aSubject == "object") && (typeof aSubject.wrappedJSObject == "function"))
				unload(aSubject.wrappedJSObject);
			break;
		}
	},
  QueryInterface: XPCOMUtils.generateQI([Ci.nsISupportsWeakReference, Ci.nsIObserver])
};

function unloadModules() { 
	Cu.unload("chrome://dslrnotifier2/content/modules/addonInfo.jsm");
	Cu.unload("chrome://dslrnotifier2/content/modules/alerts.jsm");
	Cu.unload("chrome://dslrnotifier2/content/modules/dslreports.jsm");
	Cu.unload("chrome://dslrnotifier2/content/modules/logger.jsm");
	Cu.unload("chrome://dslrnotifier2/content/modules/poller.jsm");
	Cu.unload("chrome://dslrnotifier2/content/modules/prefs.jsm");
	Cu.unload("chrome://dslrnotifier2/content/modules/privacy.jsm");
	Cu.unload("chrome://dslrnotifier2/content/modules/status.jsm");
}

function install(params, reason) {}
function uninstall(params, reason) {
	// remove preferences when uninstalling
	if (reason == ADDON_UNINSTALL) {
		Services.prefs.deleteBranch(PREFBRANCH);
	}
}

function startup(params, reason)
{
	disabled = false;

	// If installing set default button position to the nav-bar before the bookmark container
	if (reason == ADDON_INSTALL) {
		setDefaultPosition("dslr-toolbar-item", "nav-bar", "bookmarks-menu-button-container");
	};

	// Add observer for unload notifications
	Services.obs.addObserver(observer, "dslr-notifier-unload", true);
	
	// Store Addon info
	Cu.import("chrome://dslrnotifier2/content/modules/addonInfo.jsm");
	AddonInfo.addonData = [params, PREFBRANCH];

	// Read in default prefs
	Cu.import("chrome://dslrnotifier2/content/modules/prefs.jsm");
	
	// Initialize DSLReports
	Cu.import("chrome://dslrnotifier2/content/modules/dslreports.jsm");
	
	// Add unload functions
	unload(function() { 
		// If options window open, close it
		let win = Services.wm.getMostRecentWindow("Dslrnotifier2:Options");
		if (win)
			win.close();
		Services.obs.removeObserver(observer, "dslr-notifier-unload"); 
		// Unload stylesheet
		var sss = Cc["@mozilla.org/content/style-sheet-service;1"].getService(Ci.nsIStyleSheetService);
		var uri = Services.io.newURI("chrome://dslrnotifier2/skin/dslrStyle.css", null, null);
		if (sss.sheetRegistered(uri, sss.USER_SHEET))
			sss.unregisterSheet(uri, sss.USER_SHEET);	
		Services.strings.flushBundles();
	});

	// Load Style Sheet
	var sss = Cc["@mozilla.org/content/style-sheet-service;1"].getService(Ci.nsIStyleSheetService);
	var uri = Services.io.newURI("chrome://dslrnotifier2/skin/dslrStyle.css", null, null);
	sss.loadAndRegisterSheet(uri, sss.USER_SHEET);
	
	// Read in window overlay
	loadOverlay();
	
	// Watch for existing and newly opened windows
	watchWindows(applyOverlay, "navigator:browser");
}

function shutdown(params, reason)
{
	disabled = true;

	// Don't both unloading if shutting down
	if (reason == APP_SHUTDOWN)
		return;

	unload();
	
	// Unload modules after all other unloaders have run
	unloadModules();
}
