var EXPORTED_SYMBOLS = ["Privacy"];

const {classes: Cc, interfaces: Ci, utils: Cu} = Components;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");

XPCOMUtils.defineLazyGetter(this, "PrivateBrowsing", function() {
	if (Cc["@mozilla.org/privatebrowsing;1"]) {
		return Cc["@mozilla.org/privatebrowsing;1"].getService(Ci.nsIPrivateBrowsingService);
	}
	else 
		return null;
});

// This only exists in Gecko 17 and up, so define our own lazy getter to set it to null if it doesn't exist instead of throwing an exception
XPCOMUtils.defineLazyGetter(this, "PrivateBrowsingUtils", function() {
	XPCOMUtils.defineLazyModuleGetter(this, "PrivateBrowsingUtils", "resource://gre/modules/PrivateBrowsingUtils.jsm");
	try {
		return PrivateBrowsingUtils;
	}
	catch(ex) {
		return null
	}
});

// This only exists in Firefox 20 and up, so define our own lazy getter to set it to null if it doesn't exist instead of throwing an exception
XPCOMUtils.defineLazyGetter(this, "RecentWindow", function() {
	XPCOMUtils.defineLazyModuleGetter(this, "RecentWindow", "resource:///modules/RecentWindow.jsm");
	try {
		return RecentWindow;
	}
	catch(ex) {
		return null
	}
});

var Privacy = {
	isPermanentPrivateBrowsing: function() {
		return !!(PrivateBrowsingUtils && PrivateBrowsingUtils.permanentPrivateBrowsing) || !!(PrivateBrowsing && PrivateBrowsing.autoStarted);
	},

	// Unless in permanent private browsing, return a public window.
	// If aOpen flag is true, open a window if can't find or open any.
	// If aAnyWindow flag is true, get any open window
	getWindow: function(aOpen, aAnyWindow) {
		var options = aAnyWindow ? undefined : { private : this.isPermanentPrivateBrowsing() };
		var win = RecentWindow ? RecentWindow.getMostRecentBrowserWindow(options)
													 : Services.wm.getMostRecentWindow('navigator:browser');
													 
		// If no windows non-private windows open, but private windows open, open a public window
		let privWin;
		if (!win && !aAnyWindow && RecentWindow && !this.isPermanentPrivateBrowsing() && (privWin = RecentWindow.getMostRecentBrowserWindow()))
			win = privWin.OpenBrowserWindow();
			
		// Open a new window if requested
		if (!win && aOpen) {
			let args = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
			args.data = "";
			win = Services.ww.openWindow(null, Services.prefs.getCharPref("browser.chromeURL"), "_blank", "chrome,dialog=no,all", args);
		}

		return win;
	}
};