"use strict";

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
Components.utils.import("resource://gre/modules/Services.jsm");

XPCOMUtils.defineLazyModuleGetter(this, "dslrStatus", "chrome://dslrnotifier2/content/modules/status.jsm");
XPCOMUtils.defineLazyModuleGetter(this, "Prefs", "chrome://dslrnotifier2/content/modules/prefs.jsm");
XPCOMUtils.defineLazyModuleGetter(this, "dslrPoller", "chrome://dslrnotifier2/content/modules/poller.jsm");
XPCOMUtils.defineLazyModuleGetter(this, "logger", "chrome://dslrnotifier2/content/modules/logger.jsm");

const {classes: Cc, interfaces: Ci, utils: Cu} = Components;		
const OBSERVING = ["DSLRNotifier:status-change"];
const PREFS_OBSERVING = ["icoPref", "hideStatusbarIcons", "pollIm", "pollTop", "showText"];

var stringBundle = Services.strings.createBundle("chrome://dslrnotifier2/locale/dslrnotifier2.properties");

exports.loadBrowserWindow = function(window) {
		
	if (!window.gBrowser) {
		// This causes SeaMonkey to initialize window.window.gBrowser.
		window.getBrowser();
	}
	
	var dslrNotifier = {
		dslrAct: function(action,type,kill,noreverse) {
			return Private.dslrAct(action,type,kill,noreverse);
		},
		
		dslrPollIt: function() {
			return dslrPoller.checkNow();
		},
		
		openOptions: function() {
			return Private.openOptions();
		}
	};
	
	var Private = {
		load: function () {
			window.addEventListener("aftercustomization", this.updateIcons, false);
			
			OBSERVING.forEach(function(aTopic) {
				Services.obs.addObserver(this, aTopic, false);
			}, this);

			// Add preference listener
			Prefs.addListener(this);
			
			this.updateIcons();
			
			if (dslrStatus.showLoginPageOnWindowOpen) {
				dslrStatus.showLoginPageOnWindowOpen = false;
				// delay opening tab since trying this now doesn't actually load the tab
				let runnable = {
					run: function()
					{
						Private.dslrAct('tab','login','',true);
					}
				};
				Services.tm.currentThread.dispatch(runnable, Ci.nsIEventTarget.DISPATCH_NORMAL);
			}
		},
	
		unload: function() {
			window.removeEventListener("aftercustomization", this.updateIcons, false);
			
			// Remove preference listener
			Prefs.removeListener(this);
			
			// This throws sometimes because observer may be gone
			OBSERVING.forEach(function(aTopic) {
				try {
					Services.obs.removeObserver(this, aTopic);
				}
				catch(ex) { Cu.reportError(ex); }
			}, this);
			
			// Remove module and window references
			delete window.gDslrNotifier;
		},
		
		observe: function(aSubject, aTopic, aData) 
		{
			switch (aTopic)
			{
			case "DSLRNotifier:status-change":
				this.updateIcons();
				break;
			case "nsPref:changed":
				if (PREFS_OBSERVING.indexOf(aData) != -1)
					this.updateIcons();
				break;
			}
		},
		
		openOptions: function() 
		{
			let dialog = Services.wm.getMostRecentWindow("Dslrnotifier2:Options");
			if (dialog)
			{
				dialog.focus();
				return;
			}
			window.openDialog('chrome://dslrnotifier2/content/options.xul','DSLRPref','chrome,toolbar,centerscreen,resizable,dialog=yes');
		},

		updateToolbars: function(aToolbarItem, aStatusbarItem, aClass) {
			var tooltiptext = "", label = "", shIco = Prefs.icoPref;
			var showToolbarItem = (shIco != 2);
			
			switch(aClass) {
				case "dslrtoolnorm":
					tooltiptext = stringBundle.GetStringFromName("dslr.name");
					showToolbarItem = (shIco == 0);
					break;
				case "dslrtoollogin":
					tooltiptext = dslrStatus.errStr;
					break;
				case "dslrtoolim":
					tooltiptext = dslrStatus.imText + " " +  stringBundle.GetStringFromName((dslrStatus.imText != "1") ? "text.ims" : "text.im");
					label = dslrStatus.imText;
					break;
				case "dslrtoolnews":
					tooltiptext = dslrStatus.newsText + " " +  stringBundle.GetStringFromName((dslrStatus.newsText != "1") ? "text.tops" : "text.top");
					label = dslrStatus.newsText;
					break;
				case "dslrtoolhybrid":
					tooltiptext = dslrStatus.imText + " " +  stringBundle.GetStringFromName((dslrStatus.imText != "1") ? "text.ims" : "text.im") + " " +
												stringBundle.GetStringFromName("text.and") + " " + 
												dslrStatus.newsText + " " +  stringBundle.GetStringFromName((dslrStatus.newsText != "1") ? "text.tops" : "text.top");
					label = dslrStatus.imText + "/" + dslrStatus.newsText;
					break;
			}

			if (aToolbarItem) {
				aToolbarItem.setAttribute('class', "toolbarbutton-1 chromeclass-toolbar-additional " + aClass);
				aToolbarItem.setAttribute('tooltiptext', tooltiptext);
				aToolbarItem.setAttribute('count', label);
				aToolbarItem.setAttribute('notext', !Prefs.showText);
				aToolbarItem.setAttribute("error", dslrStatus.errorFlag ? "true" : "false");
				aToolbarItem.setAttribute('collapsed', showToolbarItem ? 'false' : 'true');
			}
			if (aStatusbarItem) {
				aStatusbarItem.setAttribute('class', aClass);
				aStatusbarItem.setAttribute('tooltiptext', tooltiptext);
				aStatusbarItem.setAttribute('label', label);
				aStatusbarItem.setAttribute('notext', !Prefs.showText);
				aStatusbarItem.setAttribute("error", dslrStatus.errorFlag ? "true" : "false");
				aStatusbarItem.setAttribute('collapsed', showToolbarItem ? 'false' : 'true');
			}
		},
		
		updateIcons: function() {
			var topChange = dslrStatus.newsFlag && Prefs.pollTop;
			var imChange = dslrStatus.imFlag && Prefs.pollIm;
			var logChange = dslrStatus.loginFlag;
			
			var hideStatusBar = (Services.appinfo.name == "Firefox") || Prefs.hideStatusbarIcons;
			
			// Collapse status bar icon if hidding status bar icon
			window.document.getElementById("dslr-statusbar-item").setAttribute('collapsed', hideStatusBar?'true':'false');
			
			var statusbarItem = !hideStatusBar && window.document.getElementById("dslr-statusbar-item");
			var toolbarItem = window.document.getElementById("dslr-toolbar-item");
			
			// Check if the toolbar icon exists or statusbarItem is enabled and exists
			if (toolbarItem || statusbarItem) {
				// Error notification
				if (logChange) {
					Private.updateToolbars(toolbarItem, statusbarItem, 'dslrtoollogin');
				}	
				else {
					// Hybrid IM/Topic notification
					if (topChange && imChange) 
						Private.updateToolbars(toolbarItem, statusbarItem, 'dslrtoolhybrid');
					// IM notification
					else if (imChange) 
						Private.updateToolbars(toolbarItem, statusbarItem, 'dslrtoolim');
					// Topic notification
					else if (topChange)
						Private.updateToolbars(toolbarItem, statusbarItem, 'dslrtoolnews');
					// None
					else 
						Private.updateToolbars(toolbarItem, statusbarItem, 'dslrtoolnorm');
				}
			}
		},

		dslrAct: function(action,type,kill,noreverse) {
			// Use secure site for login
			var siteName = ((type == "login") ? "https://www." : "http://www.") + Prefs.siteName;
			var url1, url2;
		
			switch (type) {
				case "im":
					url1 = siteName+"/instant?unread=1";
					if(kill) {
						dslrStatus.imFlag = false;
						dslrStatus.impoponce = false;
						dslrStatus.imslideonce = false;
						dslrStatus.imringOnce = false;
					}
					break;
				case "news":
					url1 = siteName+"/postlist?cat=umarked";
					if(kill) {
						dslrStatus.newsFlag = false;
						dslrStatus.toppoponce = false;
						dslrStatus.topslideonce = false;
						dslrStatus.tabringOnce = false; 
					}
					break;
				case "login":
					if (!dslrStatus.errorFlag) {
						url1 = siteName+"/login";
						if(kill) {
							dslrStatus.loginFlag = false; 
						}
					}
					break;
				case "menu":
					var path = Prefs.optSiteName;
					url1 = siteName+(path?path:"/");
					break;
				case "toplist":
					url1 = siteName+"/postlist?cat=marked";
					break;
				case "hybrid":
					url1 = siteName+"/instant?unread=1";
					url2 = siteName+"/postlist?cat=umarked";
					if(kill) {
						dslrStatus.newsFlag = false;
						dslrStatus.toppoponce = false;
						dslrStatus.topslideonce = false;
						dslrStatus.topringOnce = false;
						dslrStatus.imFlag = false;
						dslrStatus.impoponce = false;
						dslrStatus.imslideonce = false;
						dslrStatus.imringOnce = false;
					}
					break;
			}
			
			if (url1) {
				if (!noreverse && Prefs.reverseButtons) {
					if (action == "cur")
						action = "tab";
					else if (action == "tab")
						action = "cur";
				}
			
				var tab = this.findTabWithURL(url1);
				var select = true;
				switch(action) {
				case "cur":
					select = false;
					if (!tab)
						window.loadURI(url1);
					else
						window.gBrowser.selectedTab = tab;
					break;
				case "tab":
					window.gBrowser.selectedTab = tab ? tab : window.gBrowser.addTab(url1);
					break;
				}
				if (url2) {
					tab = this.findTabWithURL(url1);
					if (!tab)
						tab = window.gBrowser.addTab(url2);
					if (select)
						window.gBrowser.selectedTab = tab;
				}
			}
			
			if (window != Services.wm.getMostRecentWindow('navigator:browser'))
				window.focus();
			
			// If anything changed kick off processing
			if (dslrStatus.changed) {
				Services.obs.notifyObservers(null, "DSLRNotifier:status-change", null);
				dslrStatus.changed = false;
			}
		},
	
		findTabWithURL: function(url) {
			var tab = null;
			// Check each tab of this browser instance
			var numTabs = window.gBrowser.browsers.length;
			for (var index = 0; index < numTabs; index++) {
				var currentBrowser = window.gBrowser.getBrowserAtIndex(index);
				if (url == currentBrowser.currentURI.spec) {
					// The URL is already opened so reload it and return it.
					tab = window.gBrowser.tabContainer.childNodes[index];
					window.gBrowser.reloadTab(tab);
					break;
				}
			}
			return tab;
		}
	};

	// Hide and protect all functions and objects in dslrNotifier object;
	let keys = Object.keys(dslrNotifier);
	for (var i in keys) {
		Object.defineProperty(dslrNotifier, keys[i], {
			configurable: false, enumerable: false,
			value: dslrNotifier[keys[i]], 
			writable: (typeof dslrNotifier[keys[i]] != "function")
		});
	}
	
	// Expose object to Window
	window.gDslrNotifier = dslrNotifier;

	// Load the window
	Private.load.bind(Private)();
	
	// if aEvent is set that means this is being called because of a window unload, otherwise add-on in being disabled
	unload(function () {
		this.unload();
	}.bind(Private), window);
}
